package com.abi.qms.platform.service.impl;

import com.abi.qms.platform.dao.entity.BaseActive;
import com.abi.qms.platform.dao.entity.BaseMaterial;
import com.abi.qms.platform.dao.entity.BaseSapMaterial;
import com.abi.qms.platform.dao.enums.*;
import com.abi.qms.platform.dao.mapper.BaseActiveMapper;
import com.abi.qms.platform.dao.mapper.BaseMaterialMapper;
import com.abi.qms.platform.dao.mapper.BaseSapMaterialMapper;
import com.abi.qms.platform.dao.vo.result.MaterialVO;
import com.abi.qms.platform.dto.req.*;
import com.abi.qms.platform.dto.res.*;
import com.abi.qms.platform.infrastructure.util.AssertUtil;
import com.abi.qms.platform.infrastructure.util.PageUtil;
import com.abi.qms.platform.service.MaterialService;
import com.abi.task.common.api.exception.BusinessException;
import com.abi.task.common.api.exception.ErrorCodeEnum;
import com.abi.task.common.excel.preperties.MaterialProperty;
import com.abi.task.common.utils.PojoConverterUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * <p>
 * 物料
 * </p>
 *
 * @author Andy.Tan
 */
@Service
public class MaterialServiceImpl implements MaterialService {

    @Autowired
    private BaseMaterialMapper baseMaterialMapper;

    @Autowired
    private BaseSapMaterialMapper baseSapMaterialMapper;

    @Autowired
    private BaseActiveMapper baseActiveMapper;

    /**
     * 新增物料
     */
    @Override
    public void addMaterial(AddMaterialReq req){
        //校验入参
        //查询是否存在物料号
        QueryWrapper<BaseMaterial> materialQw = new QueryWrapper<>();
        materialQw.eq("material_code",req.getMaterialCode());
        materialQw.eq("is_delete", 0);
        List<BaseMaterial> materialList = baseMaterialMapper.selectList(materialQw);
        if(CollectionUtils.isNotEmpty(materialList)){
            throw new BusinessException("物料号"+req.getMaterialCode()+"已存在，请勿重复创建");
        }
        //查询SAP物料，因为新增QMS物料需要选择SAP物料来匹配
        QueryWrapper<BaseSapMaterial> baseSapMaterialQw = new QueryWrapper<>();
        baseSapMaterialQw.eq("sap_material_id",req.getMaterialCode());
        baseSapMaterialQw.eq("is_delete",0);
        BaseSapMaterial baseSapMaterial = baseSapMaterialMapper.selectOne(baseSapMaterialQw);
        AssertUtil.isNull(baseSapMaterial,"SAP物料不存在！");

        //1-新增
        BaseMaterial material = new BaseMaterial();
        material.setMaterialCode(req.getMaterialCode());
        material.setMaterialName(baseSapMaterial.getSapMaterialName());
        material.setMaterialType(req.getMaterialType());
        material.setDescription(req.getDescription());
        material.setActiveId(req.getActiveId());
        material.setHasQr(req.getHasQr());
        baseMaterialMapper.insert(material);

    }

    /**
     * 编辑物料
     */
    @Override
    public void updateMaterial(UpdateMaterialReq req){

        //1-查询物料
        BaseMaterial material = baseMaterialMapper.selectById(req.getId());
        AssertUtil.isNull(material,"物料不存在");

        //2-修改物料
        material.setMaterialType(req.getMaterialType());
        material.setDescription(req.getDescription());
        material.setValid(req.getValid());
        material.setActiveId(req.getActiveId());
        material.setHasQr(req.getHasQr());
        baseMaterialMapper.updateById(material);

    }

    /**
     * 分页查询物料
     */
    @Override
    public ListMaterialRes listMaterial(ListMaterialReq req){

        //1-分页查询
        IPage<MaterialVO> iPage = baseMaterialMapper.listMaterial(PageUtil.createPage(req), req);
        List<MaterialVO> materialList = iPage.getRecords();

        //封装出参
        ListMaterialRes res = new ListMaterialRes();
        //放入分页信息
        PageUtil.copyPageInfo(res, iPage);
        //放入出参列表
        List<ListMaterialRes.MaterialBean> materialBeanList = PojoConverterUtils.copyList(materialList, ListMaterialRes.MaterialBean.class);
        res.setMaterialBeanList(materialBeanList);

        return res;

    }

    /**
     * 查询物料明细
     */
    @Override
    public GetMaterialRes getMaterial(GetMaterialReq req){

        //1-查询物料对象
        BaseMaterial material = baseMaterialMapper.selectById(req.getId());
        AssertUtil.isNull(material, "物料不存在");

        //转化出参
        GetMaterialRes res = PojoConverterUtils.copy(material, GetMaterialRes.class);

        return res;
    }

    /**
     * 物料启用
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void enableMaterial(EnableMaterialReq req){
        List<Long> ids = req.getIds();

        //循环启用
        for (Long id : ids) {
            //1-查询活动对象
            BaseMaterial material = baseMaterialMapper.selectById(id);
            AssertUtil.isNull(material, "物料不存在");

            if (!ValidEnum.NOT_VALID.is(material.getValid())) {
                continue;
            }

            //2-修改状态
            material.setValid(ValidEnum.VALID.getCode());
            baseMaterialMapper.updateById(material);
        }
    }

    /**
     * 物料禁用
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void disableMaterial(DisableMaterialReq req){
        List<Long> ids = req.getIds();

        //循环禁用
        for (Long id : ids) {
            //1-查询活动对象
            BaseMaterial material = baseMaterialMapper.selectById(id);
            AssertUtil.isNull(material, "物料不存在");

            if (!ValidEnum.VALID.is(material.getValid())) {
                continue;
            }

            //2-修改状态
            material.setValid(ValidEnum.NOT_VALID.getCode());
            baseMaterialMapper.updateById(material);
        }
    }

    /**
     * 下拉选择物料列表
     **/
    @Override
    public ListMaterialForSelectRes listMaterialForSelect(ListMaterialForSelectReq req) {
        //1-查询所有有效的物料
        QueryWrapper<BaseMaterial> materialQw = new QueryWrapper<>();
        materialQw.eq("has_qr",1);
        materialQw.eq("valid",1);
        materialQw.eq("is_delete",0);
        List<BaseMaterial> baseMaterialList = baseMaterialMapper.selectList(materialQw);

        //构造出参
        List<ListMaterialForSelectRes.MaterialBean> materialBeanList = PojoConverterUtils.copyList(baseMaterialList, ListMaterialForSelectRes.MaterialBean.class);
        ListMaterialForSelectRes res = new ListMaterialForSelectRes();
        res.setMaterialBeanList(materialBeanList);

        return res;
    }

    /**
     * 批量导入物料
     * @param materialPropertyList
     * @return
     * @throws Exception
     */
    @Override
    public ExcelImportRes materialExcelImport(List<MaterialProperty> materialPropertyList) throws Exception {
        List<ExcelImportRes.FailBean> failList = new ArrayList<>();

        //1-循环导入，有异常则捕获
        for (int i = 0; i < materialPropertyList.size(); i++) {
            MaterialProperty materialProperty = materialPropertyList.get(i);
            try {
                //1-校验必填与枚举
                Map<String, Object> returnMap = checkMaterialProperty(materialProperty);
                String errorMessage = String.valueOf(returnMap.get("errorMessage"));
                if (StringUtils.isNotBlank(errorMessage)) {
                    throw new BusinessException(ErrorCodeEnum.ERROR_PARAM.getCode(), errorMessage);
                }

                //2-查询活动
                Long activeId = null;
                if(StringUtils.isNotBlank(materialProperty.getActiveName())){
                    QueryWrapper<BaseActive> activeQw = new QueryWrapper<>();
                    activeQw.eq("active_name",materialProperty.getActiveName());
                    activeQw.eq("is_delete",0);
                    activeQw.eq("valid",1);
                    activeQw.eq("audit_status",2);
                    BaseActive baseActive = baseActiveMapper.selectOne(activeQw);
                    AssertUtil.isNull(baseActive,"活动不存在或活动未启用");
                    activeId = baseActive.getId();
                }

                //3-调用业务代码新增
                AddMaterialReq req = new AddMaterialReq();
                req.setMaterialCode(materialProperty.getMaterialCode());
                req.setMaterialType(MaterialTypeEnum.getCode(materialProperty.getMaterialType()));
                req.setDescription(materialProperty.getDescription());
                req.setActiveId(activeId);
                req.setHasQr(TrueFalseEnum.getCode(materialProperty.getHasQr()));

                addMaterial(req);

            } catch (BusinessException e) {
                failList.add(new ExcelImportRes.FailBean(i + 2, e.getMessage()));
            }
        }

        //2-返回导入结果
        ExcelImportRes res = new ExcelImportRes();
        res.setFailedCount(failList.size());
        res.setSuccessCount(materialPropertyList.size() - res.getFailedCount());
        res.setFailBeanList(failList);

        return res;
    }

    private Map<String,Object> checkMaterialProperty(MaterialProperty materialProperty) {
        Map<String,Object> returnMap = new HashMap<>();
        StringBuffer errorMessage = new StringBuffer();
        //逐个字段校验
        if (StringUtils.isBlank(materialProperty.getMaterialCode())) {
            errorMessage.append("物料号").append("为空,");
        }
        if (StringUtils.isBlank(materialProperty.getMaterialType())) {
            errorMessage.append("物料类型").append("为空,");
        }
        if (StringUtils.isNotBlank(materialProperty.getMaterialType())
                && MaterialTypeEnum.getCode(materialProperty.getMaterialType()) == null) {
            errorMessage.append("物料类型").append("枚举值不正确,");
        }
        if(StringUtils.isBlank(materialProperty.getHasQr())){
            errorMessage.append("二维码").append("为空,");
        }
        if(StringUtils.isNotBlank(materialProperty.getHasQr())
                && TrueFalseEnum.getCode(materialProperty.getHasQr())==null){
            errorMessage.append("二维码").append("枚举值不正确,");
        }

        returnMap.put("errorMessage",errorMessage.toString());
        return returnMap;
    }

    /**
     * code查询物料
     * @param materialCode
     * @return
     */
    @Override
    public BaseMaterial getMaterialByCode(String materialCode) {

        QueryWrapper<BaseMaterial> materialQw = new QueryWrapper<>();
        materialQw.eq("material_code",materialCode);
        materialQw.eq("is_delete",0);
        BaseMaterial material = baseMaterialMapper.selectOne(materialQw);

        return material;
    }
}
